@@ -1,8 +1,8 @@
-/* A Bison parser, made by GNU Bison 3.0.4.  */
+/* A Bison parser, made by GNU Bison 3.0.5.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -303,7 +303,12 @@ extern int yydebug;
     VAR_STUB_NO_CACHE = 513,
     VAR_LOG_SERVFAIL = 514,
     VAR_DENY_ANY = 515,
-    VAR_UNKNOWN_SERVER_TIME_LIMIT = 516
+    VAR_UNKNOWN_SERVER_TIME_LIMIT = 516,
+    VAR_LOG_TAG_QUERYREPLY = 517,
+    VAR_STREAM_WAIT_SIZE = 518,
+    VAR_TLS_CIPHERS = 519,
+    VAR_TLS_CIPHERSUITES = 520,
+    VAR_TLS_SESSION_TICKET_KEYS = 521
   };
 #endif
 /* Tokens.  */
@@ -566,17 +571,22 @@ extern int yydebug;
 #define VAR_LOG_SERVFAIL 514
 #define VAR_DENY_ANY 515
 #define VAR_UNKNOWN_SERVER_TIME_LIMIT 516
+#define VAR_LOG_TAG_QUERYREPLY 517
+#define VAR_STREAM_WAIT_SIZE 518
+#define VAR_TLS_CIPHERS 519
+#define VAR_TLS_CIPHERSUITES 520
+#define VAR_TLS_SESSION_TICKET_KEYS 521
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 
 union YYSTYPE
 {
-#line 66 "util/configparser.y" /* yacc.c:1909  */
+#line 66 "util/configparser.y" /* yacc.c:1910  */
 
 	char*	str;
 
-#line 580 "util/configparser.h" /* yacc.c:1909  */
+#line 590 "util/configparser.h" /* yacc.c:1910  */
 };
 
 typedef union YYSTYPE YYSTYPE;
